//
//  ViewController.m
//  GeoCoderExample
//
//  Created by sid on 01/10/12.
//
//

#import "ViewController.h"
#import "GeoObject.h"

@interface NSString (Substring)

- (BOOL)contains:(NSString *)str;

@end

@implementation NSString (Substring)

- (BOOL)contains:(NSString *)str
{
    NSRange textRange = [[self lowercaseString] rangeOfString:[str lowercaseString]];
    
    if(textRange.location != NSNotFound)
        return YES;
    else
        return NO;
}

@end

@interface ViewController ()

@end

@implementation ViewController
@synthesize geoCodeButton;
@synthesize locationLabel;
@synthesize locationManager;
@synthesize geoCoder;

- (void)viewDidLoad
{
    [super viewDidLoad];
    locationManager.delegate=self;
    [locationManager startUpdatingLocation];
}

- (void)viewDidUnload
{
    [self setGeoCodeButton:nil];
    [self setLocationLabel:nil];
    [self setLocationManager:nil];
    [self setGeoCoder:nil];
    [super viewDidUnload];
}

- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation
{
    if ([[UIDevice currentDevice] userInterfaceIdiom] == UIUserInterfaceIdiomPhone)
    {
        return (interfaceOrientation != UIInterfaceOrientationPortraitUpsideDown);
    }
    else
    {
        return YES;
    }
}

- (IBAction)geoCodeLocation:(id)sender
{
    NSArray *results = [GeoObject parseFile];
    
    NSLog(@"results %d", [results count]);

    NSString *htmlString = @"<body><table border=\"1\">\n";
    NSString *row = @"";
    int count = 0;

    int countErrors = 0;
    int countExact = 0;
    int countNotExact = 0;
    
    [self loop:count
         array:results
        errors:countErrors
        exacts:countExact
      notExact:countNotExact
           row:row
    htmlString:htmlString];
}

- (void)loop:(int)countParam
       array:(NSArray *)resultsParam
      errors:(int)countErrorsParam
      exacts:(int)countExactParam
    notExact:(int)countNotExactParam
         row:(NSString *)rowParam
  htmlString:(NSString *)htmlStringParam
{
    __block NSString *htmlString = htmlStringParam;
    __block NSString *row = rowParam;
    __block int count = countParam;
    
    __block int countErrors = countErrorsParam;
    __block int countExact = countExactParam;
    __block int countNotExact = countNotExactParam;
    
    NSArray *results = resultsParam;
    
    NSDictionary *object = (NSDictionary *)[results objectAtIndex:count];

    double latitude = [[object valueForKey:@"latitude"] doubleValue];
    double longitude = [[object valueForKey:@"longitude"] doubleValue];
    
    CLLocation *location = [[CLLocation alloc] initWithLatitude:latitude
                                                      longitude:longitude];

    CLGeocoder *geocoder = [[CLGeocoder alloc] init];
    
    [geocoder reverseGeocodeLocation:location completionHandler:^(NSArray *placemarks, NSError *error) {
        
        BOOL hasError = NO;
        
        NSString *rowTemp = @"";
        
        if (count == 0)
            rowTemp = @"<td><b>Items</b></td>\n";
        else
            rowTemp = [rowTemp stringByAppendingFormat:@"<td><b>%d</b></td>\n", count];
        
        if (count == 0)
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Query</b></td>\n"];
        else
        {
            NSString *query = [NSString stringWithFormat:@"%@, %@, %@", [object valueForKey:@"community"], [object valueForKey:@"province"], [object valueForKey:@"city"]];
            
            rowTemp = [rowTemp stringByAppendingFormat:@"<td>%@</td>\n", query];
        }
        
        if (count == 0)
        {
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Longitude/Latitude</b></td>\n"];
        }
        else
        {
            if ([placemarks count] > 0)
            {
                CLPlacemark *placemark = (CLPlacemark *)[placemarks objectAtIndex:0];
                
                rowTemp = [rowTemp stringByAppendingFormat:@"<td>%.10f / %.10f</td>\n", placemark.location.coordinate.latitude, placemark.location.coordinate.longitude];
            }
            else
            {
                rowTemp = [rowTemp stringByAppendingString:@"<td></td>\n"];
            }
        }

        if (count == 0)
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Error</b></td>\n"];
        else
        {
            if (error)
            {
//                NSString *err = [NSString stringWithFormat:@"<td>%@</td>", [error localizedDescription]];
//     
//                rowTemp = [rowTemp stringByAppendingString:err];
                hasError = YES;
                countErrors++;
                
                NSLog(@"reintento");
                
                [self loop:count
                     array:results
                    errors:countErrors
                    exacts:countExact
                  notExact:countNotExact
                       row:row
                htmlString:htmlString];
            }
            else
            {
                rowTemp = [rowTemp stringByAppendingString:@"<td></td>\n"];
                hasError = NO;
            }
        }
        
        NSString *locatedAt = @"";
        
        if (count == 0)
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Placemark</b></td>\n"];
        else
        {
            CLPlacemark *placemark = [placemarks objectAtIndex:0];
            
            rowTemp = [rowTemp stringByAppendingFormat:@"<td>%@</td>\n", (hasError) ? @"" : placemark];
            
            locatedAt = [[placemark.addressDictionary valueForKey:@"FormattedAddressLines"] componentsJoinedByString:@", "];
        }
        
        if (count == 0)
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Forward Geocode</b></td>\n"];
        else
        {
            rowTemp = [rowTemp stringByAppendingFormat:@"<td>%@</td>\n", (hasError) ? @"" : locatedAt];
            
            if (!hasError)
            {
                if ([locatedAt contains:@"spain"])
                    countExact++;
                else
                    countNotExact++;
            }
        }
        
        if (count == 0)
            rowTemp = [rowTemp stringByAppendingString:@"<td><b>Latitude/Longitude Database</b></td>\n"];
        else
        {
            rowTemp = [rowTemp stringByAppendingFormat:@"<td>%.10f / %.10f</td>\n", latitude, longitude];
        }
        
        if (rowTemp != @"")
        {
            row = [row stringByAppendingFormat:@"<tr>%@</tr>", rowTemp];
        }
        
        [locationLabel setText:locatedAt];
        
        count++;
        
        if (count == [results count])
        {
            htmlString = [htmlString stringByAppendingFormat:@"%@", row];
            htmlString = [htmlString stringByAppendingString:@"</table>\n"];
            htmlString = [htmlString stringByAppendingString:[NSString stringWithFormat:@"<br/>Aproximate: <b>%d</b><br/>Correct: <b>%d</b><br/>Error: <b>%d</b>", countNotExact, countExact, countErrors]];
            
            htmlString = [htmlString stringByAppendingString:@"</body>"];
            
            NSString *filePath = [NSHomeDirectory() stringByAppendingPathComponent:@"Documents/report.html"];
            
            NSLog(@"FILEPATH: %@", filePath);
            
            if ([htmlString writeToFile:filePath atomically:YES encoding:NSUTF16StringEncoding error:nil])
            {
                NSLog(@"Success!!!");
            }
            else
            {
                NSLog(@"FAIL!!!");
            }
        }
        else
        {
            NSLog(@"pasa %d", count);
            
            sleep(2);
            
            [self loop:count
                 array:results
                errors:countErrors
                exacts:countExact
              notExact:countNotExact
                   row:row
            htmlString:htmlString];
        }
    }];
}

@end